from abc import abstractmethod,ABCMeta


class CustomizeHandleProcessNotFoundError(KeyError):
    pass


class BaseHandler(metaclass=ABCMeta):

    @abstractmethod
    def handle(self, content_type: str) -> callable:
        if content_type in self.__init__handle_dict:
            return self.__init__handle_dict[content_type]
        raise CustomizeHandleProcessNotFoundError(f'未找到自定义处理器{content_type}')

    def __init__(self, handle_dict: dict):
        self.__init__handle_dict = handle_dict



# encoding: utf-8
from dophon.tools import is_windows
from dophon import boot
import logging

import re

import os
import socket

import sys
import threading

import time
from urllib import request


def read_self_prop():
    try:
        def_prop = __import__('dophon.def_prop.default_properties', fromlist=True)
        u_prop = __import__('application', fromlist=True)
        # 对比配置文件
        for name in dir(def_prop):
            if re.match('__.*__', name):
                continue
            if name in dir(u_prop):
                continue
            setattr(u_prop, name, getattr(def_prop, name))
        sys.modules['properties'] = u_prop
        sys.modules['dophon.properties'] = u_prop
    except Exception as e:
        logging.error(e)
        sys.modules['properties'] = def_prop
        sys.modules['dophon.properties'] = def_prop


try:
    read_self_prop()
except Exception as e:
    logging.error('没有找到自定义配置:(application.py)')
    logging.error('引用默认配置')

from dophon import properties
from dophon_logger import *

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())


def IsOpen(ip, port):
    """
    检查端口是否被占用
    :param ip:
    :param port:
    :return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((ip, int(port)))
        logger.error('端口被占用:' + port)
        s.close()
        return True
    except:
        return False


def listen(code):
    """
    监听命令状态码
    :param code: os.system()命令返回码
    :return: 当不正常执行会抛出错误,谨慎使用!!!
    """
    if code >> 8 is 0:
        pass
    else:
        raise Exception('命令执行错误!')


def listen_container_status(container_port, loop_count: int = 3, wait_sec: int = 10):
    """
    检测容器端口存活
    :return:
    """
    # 一分钟后检测
    time.sleep(60)
    # 默认检测三次
    curr_count = 1
    while int(curr_count) <= int(loop_count):
        # 默认间隔10秒
        time.sleep(wait_sec)
        if IsOpen(get_docker_address(), int(container_port)):
            raise Exception('端口映射异常')
        else:
            # 报错证明端口正常占用
            # 发起请求
            url = f'http://{get_docker_address()}:{container_port}/rule/json'
            logger.info('容器存活性检查:' + url)
            res = request.urlopen(url)
            if not res.read():
                raise Exception('服务启动异常')
        curr_count += 1
    logger.info('容器启动成功,请在命令行输入docker ps查看')


def get_docker_address():
    """
    获取容器载体ip
    :return:
    """
    result = os.popen('ipconfig' if is_windows() else 'ifconfig').readlines()
    result_lines = []
    r_l_copy = []
    while result:
        line = result[0]
        if re.search('^.*(d|D)(o|O)(c|C)(k|K)(e|E)(r|R).*$', line):
            result_lines = result.copy()
            break
        else:
            result.pop(0)
    for line in result_lines:
        line = re.sub('\s*', '', line)
        if line and re.search('([0-9]+\.)+[0-9]+$', line) and re.search('(i|I)(p|P)', line):
            r_l_copy.append(
                re.search('([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)', line).group(0)
            )
    return r_l_copy.pop(0) if r_l_copy else 'can not get container ip'


def attach_container(base_name: str):
    """
    进入容器
    :return:
    """
    os.system('docker attach ' + base_name)


def has_version_expr(info: str):
    return re.search('(>=|==|=>|<=|=<)', info)


def run_as_docker(
        entity_file_name: str = None,
        container_port: str = str(properties.port),
        docker_port: str = str(properties.port),
        attach_cmd: bool = False,
        alive_test: bool = False,
        save_image: bool = False,
        extra_package: dict = {},
        exclude_package: list = [],
        cache_virtual_env_dir: str = '',
        package_repository: str = '',
        package_cache_path: str = '',
        timezone: str = 'Asia/Shanghai',
        img_author: str = 'local_user',
        img_message: str = 'a new image',
        container_alias: str = None,
        pip_alias: str = 'pip',
        container_pip_alias: str = 'pip',
        volumes: dict = {},
        dH: str = None,
):
    """
    利用docker启动项目
    :param save_image: 是否保存镜像(选择后不启动,直接导出镜像)
    :param alive_test: 容器存活检测(默认不检测)
    :param entity_file_name: 入口文件名(包括后缀)
    :param container_port: 容器暴露端口
    :param docker_port: 容器内部端口 -> 集群模式下的暴露端口/配置文件定义的端口
    :param attach_cmd: 是否进入容器内部sh
    :param extra_package: 额外需要加载的包以及版本
    :param exclude_package: 需要忽略的包
    :param cache_virtual_env_dir: 指定的虚拟器路径,在启动文件同级目录或下级目录
    :param package_repository: pip仓库路径(默认中央仓库)
                # 阿里云仓库  =>  https://mirrors.aliyun.com/pypi/simple/
    :param package_cache_path: 包缓存路径
    :param timezone: 时区代号(Linux时区)
    :param img_author: 镜像作者
    :param img_message: 镜像信息
    :param container_alias: 容器别名
    :param pip_alias: 宿主机pip别名
    :param container_pip_alias: 容器pip别名
    :param volumes: 容器磁盘暴露映射
    :param dH: 容器远程地址(-H)
    :return:
    """
    # 依赖包的缓存
    package_cache = []
    try:
        logger.info('容器前期准备')
        root = re.sub('\\\\', '/', properties.project_root)
        base_name = container_alias if container_alias else os.path.basename(root)
        import platform
        p_version = platform.python_version()
        work_dir = './' + base_name

        def process_prepare_file():
            # 生成依赖文件
            logger.info('生成依赖文件')
            os.system(f'{pip_alias} freeze --all >pre_requirements.txt')
            with open('./pre_requirements.txt', 'r') as file:
                with open('./requirements.txt', 'w') as final_file:

                    for line in file.readlines():
                        if line.startswith('-e'):
                            continue
                        for key in sys.modules.keys():
                            final_file_write_flag = False
                            if re.search('^dophon(_\w+)*$', key):
                                pass
                            else:
                                if re.search('^_+', key) or re.search('(_|__|\.)+.+$', key):
                                    # print(key,end=']')
                                    continue
                            module_path = re.sub(
                                '-',
                                '_',
                                re.sub('''(===|>=|==|=>|<=|=<|<|>|=)['"\w\.]+\s+''', '', line.lower())
                            )
                            # print(module_path, '===>', line, '====>', key) if line.startswith('dophon') and key.startswith(
                            #     'dophon') else None
                            if re.search(module_path.lower(), key.lower()) or re.search(key.lower(),
                                                                                        module_path.lower()):
                                if module_path in package_cache:
                                    continue
                                package_cache.append(module_path)
                                if module_path in extra_package:
                                    for exclude_package_str in exclude_package:
                                        if not re.search(f'{exclude_package_str}', module_path):
                                            final_file.write(f'{module_path}>={extra_package[module_path]}\n')
                                            final_file_write_flag = True
                                            extra_package.pop(module_path)
                                else:
                                    for exclude_package_str in exclude_package:
                                        if not re.search(f'{exclude_package_str}', module_path):
                                            final_file.write(line)
                                            final_file_write_flag = True

                                if not final_file_write_flag and not line.startswith('#'):
                                    final_file.write(line)

                                continue
                    # 写入额外包
                    for package_name, package_version in extra_package.items():
                        if package_name in extra_package:
                            final_file.write(
                                ''.join(
                                    [
                                        package_name,
                                        '' if has_version_expr(extra_package[package_name]) else '>=',
                                        extra_package[package_name],
                                        '\n']
                                )
                            )
                            # 会报迭代修改异常
                            # extra_package.popitem()
                        else:
                            final_file.write(
                                ''.join(
                                    [
                                        package_name,
                                        '' if has_version_expr(extra_package[package_name]) else '>=',
                                        package_version,
                                        '\n']
                                )
                            )

        def process_docker_file():
            # 生成Dockerfile
            logger.info('生成Dockerfile')
            with open('./Dockerfile', 'w') as file:
                file.write('FROM callmee/dophon\n')
                file.write('ADD . ' + work_dir + '\n')
                # file.write('ADD . ' + work_dir + '/' + base_name + '\n')
                file.write('WORKDIR ' + work_dir + '\n')
                for k, v in volumes.items():
                    assert k.startswith('~/') or k.startswith('/'), 'container path must starts with ~/ or /'
                    assert v.startswith('~/') or v.startswith('/'), 'host path must starts with ~/ or /'
                    file.write(f'VOLUME {k.replace("~", ".")}:{v.replace("~", ".")}\n')
                if cache_virtual_env_dir:
                    file.write(f'ADD {cache_virtual_env_dir} ~/.cache_virtual_env' + '\n')
                    file.write(f'CMD ~/.cache_virtual_env/Scripts/activate' + '\n')
                if package_cache_path:
                    file.write(f'ADD {package_cache_path} ./.package_cache' + '\n')
                    file.write(f'ENV PYTHONPATH=/' + base_name + '/.package_cache' + '\n')
                    pass
                if package_repository:
                    # 阿里云仓库  =>  https://mirrors.aliyun.com/pypi/simple/
                    file.write(f'RUN {container_pip_alias} install -i {package_repository} -r requirements.txt' + '\n')
                    pass
                else:
                    file.write(f'RUN {container_pip_alias} install --no-cache-dir -r requirements.txt' + '\n')
                    pass
                # 设置系统时区
                file.write(f'ENV TZ={timezone}' + '\n')
                file.write('RUN ln -snf /usr/share/zoneinfo/$TZ /etc/localtime && echo $TZ > /etc/timezone' + '\n')
                file.write(
                    'CMD ["python","./' + (entity_file_name if entity_file_name else 'Bootstrap.py') + '"]' + '\n')
                # file.write('CMD ["/bin/bash"]' + '\n')

        def process_docker_command():
            # 生成docker command
            docker_command = f'docker -H {dH} ' if dH else 'docker '
            os.system('cd ' + root)
            logger.info('暂停已运行的实例')
            os.system(f'{docker_command}stop {base_name}')
            logger.info('移除已运行的实例')
            os.system(f'{docker_command}rm {base_name}')
            logger.info('移除旧镜像')
            os.system(f'{docker_command}rmi {base_name}')
            logger.info('检测配置合法性')
            if IsOpen('127.0.0.1', int(docker_port)):
                # 端口被占用
                logger.warn('映射端口被占用!!')
            logger.info('建立镜像')
            listen(os.system(f'{docker_command}build -t {base_name} .'))
            logger.info('运行镜像')
            os.system(
                f'{docker_command}run -p {container_port}:{docker_port} -d --name {container_alias if container_alias else base_name} {os.path.basename(root)}'
            )
            if save_image:
                logger.info('检测容器状态')
                from subprocess import Popen, PIPE
                status_code = '\'{{.State.Status}}\''
                __status = 'running'
                while __status == status_code:
                    p = Popen(
                        f"{docker_command}inspect {base_name} -f {status_code}",
                        stdout=PIPE,
                        stderr=PIPE
                    )
                    p.wait()
                    __status = eval(p.stdout.read().decode('utf-8'))
                    from urllib import request
                    try:
                        time.sleep(5)
                        res = request.urlopen(f'http://127.0.0.1:{container_port}/rule/map')
                        # for name in dir(response):
                        #     print(f'{name}===>{getattr(response, name)}')
                        if int(res.code) == 200:
                            break
                    except Exception as e:
                        logger.error(f'{e}')
                        break
                    if status_code == 'exited':
                        raise Exception(f'容器启动失败,状态为{__status}')
                logger.info('提交镜像')
                from datetime import datetime
                __commit_image_name = f'image_{base_name}{datetime.now().timestamp()}'
                # 保存镜像
                os.system(f"""
                        {docker_command}commit --author "{img_author}" --message "{img_message}" {base_name} {__commit_image_name}
                        """)
                logger.info('生成镜像')
                os.system(f"""{docker_command}save -o {base_name}.img.bak.__ {__commit_image_name}""")
                logger.info('停止镜像')
                os.system(f"""{docker_command}stop {base_name}""")
                exit(0)
                return
            logger.info('打印容器内部地址(空地址代表启动失败)')
            os.system(f'{docker_command} inspect --format=' + '\'{{.NetworkSettings.IPAddress}}\'' + f' {base_name}')
            logger.info('打印容器载体地址')
            print(get_docker_address())
            if alive_test:
                logger.info('启动检测容器端口')
                threading.Thread(target=listen_container_status, args=(container_port,)).start()
            if attach_cmd:
                logger.info('进入镜像')
                # threading.Thread(target=attach_container,args=(base_name,)).start()
                attach_container(base_name)
            logger.info('容器启动完毕')

        logger.debug('process_prepare_file')
        process_prepare_file()
        logger.debug('process_docker_file')
        process_docker_file()
        logger.debug('process_docker_command')
        process_docker_command()
    except Exception as e:
        logger.error(e)
        raise e



# coding: utf-8
import re

from flask import Blueprint

from dophon.tools.dynamic_import import d_import

name = 'dophon'


def blue_print(name, import_name, inject_config: dict = {}, static_folder=None,
               static_url_path=None, template_folder=None,
               url_prefix=None, subdomain=None, url_defaults=None,
               root_path=None):
    """
    获取Flask路由,同时实现自动注入(可细粒度管理)
    :param inject_config:注入配置,类型为dict
    :param name:同Blueprint.name替换为--
           2021-06-02: blueprint名称不允许包含.,
    :param import_name:同Blueprint.import_name
    :param static_folder:同Blueprint.static_floder
    :param static_url_path:同Blueprint.static_url_path
    :param template_folder:同Blueprint.template_folder
    :param url_prefix:同Blueprint.url_prefix
    :param subdomain:同Blueprint.subdomain
    :param url_defaults:同Blueprint.url_defaults
    :param root_path:同Blueprint.root_path
    :return:
    """
    blue_print_obj = Blueprint(
        name=re.sub(r'\.', '--', name), import_name=import_name, static_folder=static_folder,
        static_url_path=static_url_path, template_folder=template_folder,
        url_prefix=url_prefix, subdomain=subdomain, url_defaults=url_defaults,
        root_path=root_path
    )
    if inject_config:
        autowire = __import__('dophon.annotation.AutoWired', fromlist=True)
        outerwired = getattr(autowire, 'OuterWired')
        outerwired(
            obj_obj=inject_config['inj_obj_list'],
            g=inject_config['global_obj']
        )(inject)()
    return blue_print_obj


def inject():
    """
    用于构造注入入口,无意义
    :return:
    """
    pass


def dophon_boot(f):
    """
    装饰器形式启动
    :return:
    """

    def arg(*args, **kwargs):
        from . import tools
        boot = __import__('dophon.boot', fromlist=True)
        kwargs['boot'] = boot
        return f(*args, **kwargs)

    return arg


__all__ = ['BluePrint', 'blue_print']

BluePrint = blue_print



import re
import json
from functools import wraps
from inspect import getfullargspec, signature, _empty
from dophon import properties
from dophon.tools.sys_utils import to_dict

DESC_INFO = {}
TRANSLATE_TYPES = ['int', 'set']
SPECIAL_TRANSLATE_TYPES = ['list', 'dict']
RETURN_TYPE_DESC = {
    'dict': {}, 'list': [], set: [], 'tuple': [], 'str': "\"\"", 'int': 0
}


def try_str(obj):
    try:
        return json.dumps(obj, ensure_ascii=False)
    except:
        try:
            return to_dict(obj)
        except:
            return str(obj)


def desc(
        operation: str = 'None',
        param_des: dict = {},
        demo: dict = None,
        return_demo=None,
        remark: str = None,
):
    """
    函数描述装饰器
    自动添加请求属性配对
    :return:
    """
    if properties.load_desc:
        def inner_method(f):
            # 获取方法的参数名集合
            # FullArgSpec(args=['test_arg1'], varargs=None, varkw=None, defaults=None, kwonlyargs=[], kwonlydefaults=None, annotations={})
            full_arg_spec = getfullargspec(f)
            func_return_type = signature(f).return_annotation
            func_return_type = func_return_type if func_return_type != _empty else None
            __own_doc = getattr(f, '__doc__')  # 自身定义的文档
            __args = full_arg_spec.args  # 通过函数定义的属性名集合
            __defaults = full_arg_spec.defaults  # 通过函数定义参数默认值
            __defaults = __defaults if __defaults else ()
            # 处理无默认值参数
            if __defaults and len(__args) > len(__defaults):
                __pre_defaults = list(__defaults)
                for count in range(len(__args) - len(__defaults)):
                    __pre_defaults.insert(0, None)
                __defaults = tuple(__pre_defaults)
            # print(__defaults)
            __annotations = full_arg_spec.annotations  # 通过函数定义的参数修饰
            # print(full_arg_spec)
            # 处理参数信息,放入参数信息列
            __function_arg_info = {}
            __func_info = {
                'own_doc': re.sub('(\n)', '<br />', __own_doc) if __own_doc else operation
            }
            for __inner_arg_index in range(len(__args)):
                __function_arg_type = __annotations[__args[__inner_arg_index]].__name__ \
                    if __annotations and __args[__inner_arg_index] in __annotations else 'any'
                __function_arg_dvalue = (eval(
                    f"""{__function_arg_type}""" + \
                    f"""("{__defaults[
                        __inner_arg_index]}")""") \
                    if __function_arg_type in TRANSLATE_TYPES else (__defaults[__inner_arg_index] \
                    if __function_arg_type in SPECIAL_TRANSLATE_TYPES else (try_str(__defaults[__inner_arg_index]) \
                    if __defaults and __inner_arg_index < len(
                    __defaults) else None))) \
                    if __defaults and __inner_arg_index < len(__defaults) else None
                __function_arg_info[__args[__inner_arg_index]] = {
                    'name': __args[__inner_arg_index],
                    'desc': param_des.get(__args[__inner_arg_index]) if __args[
                                                                            __inner_arg_index] in param_des else None,
                    # 获取参数上的默认值
                    # 隐藏对象地址
                    'default_value': re.sub(' at.*>', '>', __function_arg_dvalue
                                            ) if isinstance(__function_arg_dvalue, str) else __function_arg_dvalue
                    if __defaults and __inner_arg_index < len(__defaults) else None,
                    # 获取参数上的类型修饰
                    'type': __annotations[__args[__inner_arg_index]].__name__
                    if __annotations and __args[__inner_arg_index] in __annotations
                    else 'any',
                }
            __func_info['remark'] = remark
            __func_info['args_info'] = __function_arg_info
            __func_return_type_desc = func_return_type.__name__ \
                if func_return_type \
                else 'any'
            __func_info['demo'] = demo
            __func_info['returns'] = {
                'type': __func_return_type_desc,
                'demo': return_demo
                if return_demo else (RETURN_TYPE_DESC[__func_return_type_desc]
                                     if __func_return_type_desc in RETURN_TYPE_DESC else to_dict(
                    func_return_type.__init__()))
            }
            DESC_INFO[f'{getattr(f, "__module__")}.{getattr(f, "__name__")}'] = __func_info

            @wraps(f)
            def inner_args(*args, **kwargs):
                return f(*args, **kwargs)

            return inner_args

        return inner_method
    else:
        def pass_inner_method(f):
            @wraps(f)
            def pass_inner_args(*args, **kwargs):
                return f(*args, **kwargs)

            return pass_inner_args

        return pass_inner_method



# coding: utf-8
import functools
import re
from flask import request, abort
from dophon import properties
from dophon_logger import *

from dophon.errors.ParamErrors import ReqParamError
from .param_adapter.handler import BaseHandler, CustomizeHandleProcessNotFoundError
from .param_adapter import *
import inspect
import json

logger = get_logger(eval(properties.log_types))

# Content-TYPE 2 adapter
PARAM_DICT = {
    None: is_none,
    'application/x-www-form-urlencoded': is_form,
    'multipart/form-data': is_form,
    'application/json': is_json,
    'text/xml': is_xml,
    'application/xml': is_xml,
    'text/plain': is_args
}

'''
参数体可以为多个,形参名称必须与请求参数名一一对应(只少不多)
装饰器中关键字参数列表可以指定参数名
'''

logger.inject_logger(globals())

# 处理请求参数装饰器(分离参数)
true = True
false = False


def auto_param(mix: bool = False, mix_param: str = 'p', content_handler: BaseHandler = None, headers: bool = False):
    def method(f):
        final_kwargs = {}
        @functools.wraps(f)
        def args(*args, **kwargs):
            try:
                content_type = request.headers.get('Content-Type').split(';')[0]
                if content_type:
                    if not content_handler:
                        handle_result = PARAM_DICT[content_type](request)
                        req_kwargs = eval(json.dumps(handle_result, ensure_ascii=False)) \
                            if isinstance(handle_result, dict) \
                            else handle_result
                    else:
                        try:
                            # 尝试自定义处理器处理请求
                            req_kwargs = content_handler.handle(content_type)(request)
                        except CustomizeHandleProcessNotFoundError as e:
                            # 尝试使用框架自定义处理器处理
                            handle_result = PARAM_DICT[content_type](request)
                            req_kwargs = eval(json.dumps(handle_result, ensure_ascii=False)) \
                                if isinstance(handle_result, dict) \
                                else handle_result
                else:
                    req_kwargs = request.args.to_dict()

                # 整合请求路径参数
                path_req_args = request.args.to_dict()
                for path_req_k in path_req_args:
                    if path_req_k not in req_kwargs:
                        req_kwargs[path_req_k] = path_req_args[path_req_k]

                assert isinstance(req_kwargs, dict), f'参数类型异常{type(req_kwargs)}'

                if mix:
                    params = inspect.signature(f).parameters
                    kwarg_keys = params.keys()
                    if mix_param in kwarg_keys:
                        mixed_kwargs = {
                            mix_param: req_kwargs
                        }
                        for k in kwarg_keys:
                            item = params[k]
                            if k not in mixed_kwargs:
                                mixed_kwargs[k] = None if isinstance(item.default, inspect._empty) else item.default
                        final_kwargs = mixed_kwargs
                    else:
                        raise ReqParamError(f'方法{f}缺少混合参数{mix_param}')
                else:
                    params = inspect.signature(f).parameters
                    kwarg_keys = params.keys()
                    for k in kwarg_keys:
                        if k in req_kwargs:
                            kwargs[k] = req_kwargs[k]
                        else:
                            item = params[k]
                            kwargs[k] = None if isinstance(item.default, inspect._empty) else item.default
                    final_kwargs = kwargs
                    return f(**final_kwargs)

                if headers:
                    final_kwargs['headers'] = request.headers
                return f(**final_kwargs)
            except TypeError as t_e:
                logger.error('参数不匹配!!,msg:' + repr(t_e))
                raise t_e
                return abort(400)
            except AttributeError as a_e:
                logger.error('参数不匹配!!,msg:' + repr(a_e))
                return abort(400)

        return args

    return method


'''
注意!!!
该注解只需方法体内存在一个形参!!!
同样,需要指定参数名的形式参数列表条目也只能存在一个,多个会默认取第一个
不匹配会打印异常
参数以json形式赋值
'''


# 处理请求参数装饰器(统一参数,参数体内参数指向json串)
def full_param(kwarg_list=[]):
    def method(f):
        @functools.wraps(f)
        def args(*args, **kwargs):
            try:
                if 'POST' == str(request.method):
                    r_arg = ()
                    r_kwarg = {}
                    if not kwarg_list:
                        r_arg = (request.json if request.is_json else request.form.to_dict(),)
                    else:
                        r_kwarg[kwarg_list[0]] = request.json if request.is_json else request.form.to_dict()
                    return f(*r_arg, **r_kwarg)
                elif 'GET' == str(request.method):
                    r_arg = ()
                    r_kwarg = {}
                    if not kwarg_list:
                        r_arg = (request.args.to_dict(),)
                    else:
                        r_kwarg[kwarg_list[0]] = request.args.to_dict()
                    return f(*r_arg, **r_kwarg)
                else:
                    logger.error('json统一参数不支持该请求方法!!')
                    return abort(400)
            except TypeError as t_e:
                logger.error('参数不匹配!!,msg:' + repr(t_e))
                return abort(500)

        return args

    return method


def file_param(alias_name: str = 'files', extra_param: str = 'args', headers: bool= False):
    """
    文件参数装在装饰器

    ps 文件上传暂时只支持路由方法内单个参数接收(会有校验策略)

    参数demo(小程序):
        ImmutableMultiDict([('img_upload_test', <FileStorage: 'filename' ('image/jpeg')>)])
    :return:
    """

    def method(f):
        @functools.wraps(f)
        def args(*args, **kwargs):
            # 检测参数
            a_nums = len(args) + len(kwargs)
            if a_nums > 0:
                logger.error('路由绑定参数数量异常')
                raise Exception('路由绑定参数数量异常')
            try:
                extra_param_value = (request.form if request.form else request.json).to_dict()
            except:
                extra_param_value = {}
            k_args = {
                alias_name: request.files.to_dict(),
                extra_param: extra_param_value
            }
            if headers:
                k_args['headers'] = request.headers
            return f(**k_args)

        return args

    return method


# 路径绑定装饰器
# 默认服务器从boot获取
def request_mapping(path='', methods=[], app=None):
    def method(f):
        try:
            # 自动获取蓝图实例并进行http协议绑定
            current_package = __import__(str(getattr(f, "__module__")), fromlist=True)
            try:
                package_app = getattr(current_package, '__app') \
                    if hasattr(current_package, '__app') \
                    else current_package.app \
                    if hasattr(current_package, 'app') \
                    else app \
                    if hasattr(app, 'routes') \
                    else __import__('dophon').blue_print(f"_annotation_auto_reg.{getattr(current_package, '__name__')}",
                                                         getattr(current_package, '__name__'))
            except Exception as e:
                logger.warn(f'{e}')
                package_app = __import__('dophon.boot', fromlist=True).app
            # 回设内部蓝图参数
            # print(package_app)
            setattr(current_package, '__app', package_app)
            result = package_app.route(path, methods=methods)(f)
        except Exception as e:
            logger.error(f'{getattr(f, "__module__")}参数配置缺失,请检查({path},{methods},{package_app})')

        def m_args(*args, **kwargs):
            if properties.debug_trace:
                logger.info(f'router endpoint:{getattr(f, "__module__")}.{f},router path:{path}')
            return result(*args, **kwargs)

        return m_args

    return method


# 请求方法缩写
def method_route(req_method: str, path: str = '', ):
    def method(f):
        result = request_mapping(path, [req_method])(f)

        def _args(*args, **kwargs):
            return result(*args, **kwargs)

        return _args

    return method


# get方法缩写
def get_route(path=''):
    def method(f):
        result = method_route('get', path)(f)

        def _args(*args, **kwargs):
            return result(*args, **kwargs)

        return _args

    return method


# post方法缩写
def post_route(path=''):
    def method(f):
        result = method_route('post', path)(f)

        def _args(*args, **kwargs):
            return result(*args, **kwargs)

        return _args

    return method


func_to_path = lambda f: f"""{"/" if re.match("^[a-zA-Z0-9]+", getattr(f, "__name__")) else ""}{re.sub("[^a-zA-Z0-9]",
                                                                                                       "/",
                                                                                                       getattr(f,
                                                                                                               "__name__"))}"""


# get方法缩写
def get(f, *args, **kwargs):
    path = func_to_path(f)
    result = get_route(re.sub('\s+', '', path))(f)

    @functools.wraps(f)
    def method():
        def _args():
            return result(*args, **kwargs)

        return _args

    return method


# post方法缩写
def post(f, *args, **kwargs):
    path = func_to_path(f)

    result = post_route(re.sub('\s+', '', path))(f)

    @functools.wraps(f)
    def method():
        def _args():
            return result(*args, **kwargs)

        return _args

    return method



import os
from subprocess import Popen, PIPE, DEVNULL

from dophon_properties import *
from tqdm import tqdm

get_property(DOPHON)
from dophon_logger import *
from dophon import properties

get_logger(eval(properties.log_types)).inject_logger(globals())

from dophon.tools import is_windows
from importlib import util


def __dynamic_import(module_name, version='*'):
    __module = None
    while True:
        try:
            # 校验模块安装
            util.find_spec(f'{module_name}')
            # exec(f'import {module_name}')
            module = __module if __module else __import__(module_name)
            # sys.modules[module_name] = module
            logger.info(f'模块{module_name}依赖建立完毕')
            return module
        except Exception as e:
            # print(e)
            logger.info(f"install {module_name} >={version if version else 'release'}")
            # 利用pip模块安装所需模块
            pip_arg_list = ['pip', 'install',
                            module_name + (('>=' + version) if version else ''),
                            '--user']
            exe_pip_args = ['install',
                            module_name + (('>=' + version) if version else ''),
                            '--user']
            if not version:
                pip_arg_list.append('-U')
                exe_pip_args.append('-U')
            # 利用popen执行命令安装包
            # p = Popen(f'pip install {module_name} {"" if is_windows() else "--user"}', stdout=DEVNULL, stderr=DEVNULL)
            p = Popen(f'pip install {module_name} {"" if is_windows() else "--user"}',stdout=PIPE,stderr=PIPE)
            p.wait()
            # tqdm(p.stdout.readlines(), f'install {module_name}')
            lines_queue = tqdm(p.stdout.readlines())
            for item in lines_queue:
                lines_queue.set_description(item)
            __module = __import__(module_name)


def __d_import(module_name: str, version: str = '*'):
    """
    将需要安装的包信息放入全局字典中
    :param module_name:
    :param version:
    :return:
    """
    while True:
        try:
            __import__(module_name)
            break
        except Exception as e:
            # print(e)
            __dynamic_import(module_name=module_name, version=version)


def d_import(module_name, version: str = '*'):
    """
    对外暴露的动态添加函数入口
    :param module_name:
    :param version:
    :return:
    """
    __d_import(module_name=module_name, version=version)



import xmltodict


def is_none(request):
    return request.args


def is_json(request):
    return request.json


def is_form(request):
    return request.form.to_dict()


def is_args(request):
    return request.args.to_dict()


def is_xml(request):
    xml_str = str(request.data, encoding='utf8')
    target = xmltodict.parse(xml_str, encoding='utf-8')
    return target



JSONViewer = """
function formatJson (msg) {
var rep = '~';
var jsonStr = JSON.stringify(msg, null, rep)
//var matcher = jsonStr.match(/\[(\s|\w-_|'|"|,)*\]/g)
//console.debug(matcher instanceof Array,matcher)
//if(matcher && matcher instanceof Array){
//   for(let m_index = 0;m_index<matcher.length;m_index++){
//       jsonStr.replace(matcher[m_index],matcher.replace(/[\s~]*/g,''))
//   }
//}
var str = '';
for (var i = 0; i < jsonStr.length; i++) {
var text2 = jsonStr.charAt(i)
if (i > 1) {
var text = jsonStr.charAt(i - 1)
if (rep != text && rep == text2) {
//str += '<br/>'
}
}
str += text2;
}
jsonStr = '';
for (var i = 0; i < str.length; i++) {
var text = str.charAt(i);
if (rep == text) { jsonStr += '&nbsp;&nbsp;&nbsp;&nbsp;' } else {
jsonStr += text;
}
if (i == str.length - 2) { jsonStr += '' }
}
return jsonStr.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\\b(true|false|null)\\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function(match) {
                    var cls = 'number';
                    if (/^(&nbsp;)*"/.test(match)) {
                        if (/:$/.test(match)) {
                            cls = 'key';
                        } else {
                            cls = 'string';
                        }
                    } else if (/true|false/.test(match)) {
                        cls = 'boolean';
                    } else if (/null/.test(match)) {
                        cls = 'null';
                    }
                    return '<span class="' + cls + '">' + match + '</span>';
                });
}
"""


# coding: utf-8
# 自动注入修饰器
import re
import inspect
from dophon import properties
from dophon_logger import *
import types
from dophon.tools import sys_utils

logger = get_logger(eval(properties.log_types))

"""
自动注入注解

ps:默认为单例模式,利用模块局内变量域管理实例,减少内存冗余

author:CallMeE
date:2018-06-01

实现自动注入功能需在路由模块(也可以在其他位置)中定义方法(inject_obj)并添加下列两个注解之一
DEMO;
# 注入对象
@AutoWired.InnerWired([ShopGoodsController.ShopGoodsController],a_w_list=['_SGCobj'],g=globals())
或者
@AutoWired.OuterWired(obj_list,g=globals())
def inject_obj():
    pass
调用inject_obj()即可实现自动注入
注意:
1.a_w_list中的元素为注入引用名,必须要与注入目标引用名一致,否则注入失效
2.注入位置必须显式定义一个值为None的引用,否则编译不通过
3.注入类型必须为可初始化类型(定义__new__ or __init__)
"""

logger.inject_logger(globals())


class ObjManager(dict):

    def reg_bean(self, alias: str, obj: object):
        if self[alias]:
            logger.warning(f'{alias}存在,注册忽略')
        else:
            self[alias] = obj

    def __getitem__(self, item):
        logger.debug(f'获取对象{item} ===> {self.get(item)}   *后续版本删除打印')
        return self.get(item)


obj_manager = ObjManager()


class UniqueError(Exception):
    """
    唯一错误,存在覆盖已存在的别名实例
    """


# 显式参数注入
def InnerWired(clz, g, a_w_list=[]):
    # print(locals())
    # 注入实例
    def wn(f):
        def inner_function(*args, **dic_args):
            # 获取数组
            if a_w_list and 0 < len(a_w_list):
                # 装饰器参数查找赋值
                a_name = a_w_list
            else:
                if dic_args['a_w_list'] and 0 < dic_args['a_w_list']:
                    # 被装饰函数关键字参数查找赋值
                    a_name = dic_args['a_w_list']
                else:
                    if not args:
                        logger.error('动态形参为空')
                        raise Exception('动态形参为空!!')
                    # 被装饰函数位置参数查找赋值
                    a_name = args[0]
            for index in range(len(a_name)):
                logger.info(str(a_name[index]) + " 注入 " + str(clz[index]))
                obj_name = a_name[index]
                if obj_name in obj_manager:
                    g[obj_name] = obj_manager[obj_name]
                else:
                    instance = clz[index]()
                    obj_manager[obj_name] = instance
                    g[obj_name] = instance
            # return arg
            return f()

        return inner_function

    return wn


# 自定义参数列表注入
def OuterWired(obj_obj, g):
    # 前期准备
    clz = []
    a_w_list = []
    for key in obj_obj.keys():
        a_w_list.append(key)
        clz.append(obj_obj[key])

    # 注入实例
    def wn(f):
        logger.info('开始注入实例')

        def inner_function(*args, **dic_args):
            # 获取数组
            if a_w_list and 0 < len(a_w_list):
                # 装饰器参数查找赋值
                a_name = a_w_list
            else:
                if dic_args['a_w_list']:
                    # 被装饰函数关键字参数查找赋值
                    a_name = dic_args['a_w_list']
                else:
                    if not args:
                        logger.error('动态形参为空!!')
                        raise Exception('动态形参为空!!')
                    # 被装饰函数位置参数查找赋值
                    a_name = args[0]
            for index in range(len(a_name)):
                logger.info(str(a_name[index]) + " 注入 " + str(clz[index]))
                try:
                    obj_name = a_name[index]
                    if obj_name in obj_manager:
                        g[obj_name] = obj_manager[obj_name]
                    else:
                        instance = clz[index]()
                        obj_manager[obj_name] = instance
                        g[obj_name] = instance
                except Exception as e:
                    logger.error('注入' + str(a_name[index]) + '失败,原因:' + str(e))
                    continue
            return f()

        return inner_function

    return wn


"""
2018-07-22

参照spring实例管理实现实例管理
1.bean装饰器
    参照spring中bean注解
    需执行被装饰方法才能交由实例管理器管理
    默认使用方法名作为实例管理名
    可传入自定义别名替换别名
    同名别名会抛出二义性错误
    暂不支持lambda表达式(强制调用非装饰器装饰lambda表达式会产生注册实例无效)
2.BeanConfig实例管理器
    定义管理器子类后定义bean装饰方法后实例化一次后全部交由全局实例管理管理实例
    支持with语法
3.Bean实例获取类
    实例获取需传入实例别名或实例类型
    可直接赋值变量
"""


def bean(name: str = None, value=None):
    """
    向实例管理器插入实例
    :param by_name: 别名(不传值默认为类型)
    :return:
    """

    def insert_method(f):
        def args(*args, **kwargs):
            result = f(*args, **kwargs)
            if result is None:
                raise TypeError('无法注册实例:' + str(result))
            if name:
                if name in obj_manager:
                    raise UniqueError('存在已注册的实例:' + name)
                obj_manager[name] = result
            else:
                alias_name = getattr(f, '__name__') if getattr(f, '__name__') else getattr(type(result), '__name__')
                if alias_name in obj_manager:
                    raise UniqueError('存在已注册的实例')
                else:
                    obj_manager[alias_name] = result
            return

        return args

    def insert_value():
        if name:
            if name in obj_manager:
                raise UniqueError('存在已注册的实例:' + name)
            obj_manager[name] = value
        else:
            alias_name = name
            if alias_name in obj_manager:
                raise UniqueError('存在已注册的实例')
            else:
                obj_manager[alias_name] = value

    return insert_value if value else insert_method


bean_const = ['file_init', 'sort_bean_config']


class BeanConfig:
    """
    实力配置类,类内自定义方法带上bean注解即可(参照springboot中been定义)
    """

    def __call__(self, *args, **kwargs):
        for name in dir(self):
            if re.match('__.+__', name) or name in bean_const:
                continue
            attr = getattr(self, name)
            if callable(attr):
                fields = inspect.getfullargspec(attr).args
                staticmethod(attr(*fields))

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type and exc_val and exc_tb:
            logger.error(f'{str(exc_val)} 实例不存在')
        pass

    def __init__(self, files: list = [], config_file: str = ''):
        if obj_manager:
            logger.warning(f'存在已初始化实例管理器,跳过初始化: {self}')
            return
        logger.info('执行批量实例管理初始化')
        if files or config_file:
            self.file_init(files if files else [config_file])
        else:
            # 无参构造,加载内部bean装饰方法,实例统一配置
            self()

    def file_init(self, files: list = []):
        for config_file in files:
            if isinstance(config_file, str):
                # 字符串形式声明配置文件(文件名)
                logger.info('读取配置文件: %s ' % (config_file,))
                # 有参构造,分布式实例配置
                from dophon import properties
                import os
                file_path = properties.project_root + (config_file if config_file.startswith(os.sep) else (
                        os.sep + config_file))
                if os.path.exists(file_path):
                    try:
                        bean_config = __import__(config_file.replace(os.sep, '.').rstrip('.py'), fromlist=True)
                        self.sort_bean_config(bean_config)
                    except Exception as e:
                        logger.error('实例配置读取失败,信息: %s' % (e,))
                else:
                    logger.error('不存在实例配置文件')
                    self()
            elif isinstance(config_file, types.ModuleType):
                # 模块形式配置文件
                try:
                    logger.info('读取配置文件: %s ' % (config_file,))
                    self.sort_bean_config(config_file)
                except Exception as e:
                    logger.error('实例配置读取失败,信息: %s' % (e,))
            else:
                # 其他形式声明配置
                logger.error('配置文件声明无效(%s)' % (config_file,))
                continue

    def sort_bean_config(self, bean_config: object):
        for k in dir(bean_config):
            if not k.startswith('__') and not k.endswith('__'):
                # 获取自定义参数
                # print(k, '---', getattr(bean_config, k))
                bean(k, getattr(bean_config, k))()
                # print(obj_manager[k])


class Bean:
    def __new__(cls, *args, **kwargs):
        if args or len(args) > 1:
            bean_key = args[0]
        elif kwargs or len(kwargs) > 1:
            bean_key = kwargs.keys()[0]
        else:
            raise KeyError('不存在实例别名或实例类型')
        if isinstance(bean_key, str):
            if bean_key in obj_manager:
                return obj_manager[bean_key]
            raise KeyError(f'不存在该别名实例:({bean_key})')
        elif isinstance(bean_key, type):
            type_list = []
            for key in obj_manager.keys():
                if re.match('__.+__', key):
                    continue
                bean_obj = obj_manager[key]
                if isinstance(bean_obj, bean_key):
                    if len(type_list) > 0:
                        raise UniqueError(f'存在定义模糊的实例获取类型:({bean_key})')
                    type_list.append(bean_obj)
            if not type_list:
                raise KeyError(f'不存在该类型实例:({bean_key})')
            return type_list[0]
        else:
            logger.error('无法获取的实例: %s' % (bean_key,))


def DefBean(clz):
    # print(f'define bean class{clz}')
    if type(clz) is type:
        try:
            __instance = clz()
            # for name in dir(clz):
            #     print(f'{name} => {getattr(clz, name)}')
            # print(f'''{clz} => {}''')
            alias_fstr = f"_{getattr(clz, '__name__')}__bean_alias"
            clz_alias = sys_utils.to_lower_camel(getattr(clz, alias_fstr)) \
                if hasattr(clz, alias_fstr) \
                else getattr(clz, '__name__')
            if clz_alias not in obj_manager:
                obj_manager[sys_utils.to_lower_camel(clz_alias)] = __instance
        except Exception as e:
            logger.error(f'初始化{clz}失败,{e}')
    # print(obj_manager)



from flask import Response
from flask import jsonify

"""
框架自身返回结果类
"""


class JsonResult:
    """
    json类型返回结果类
    """
    __body = {}

    default_mimetype = 'application/json'

    def __init__(self, event=200, data={}, msg=''):
        self.__status = event
        self.__body['event'] = str(event)
        self.__body['data'] = str(data)
        self.__body['msg'] = str(msg)

    def as_res(self):
        return jsonify(self.__body), self.__status


class XmlResult:
    """
    XML类型返回结果类
    """
    __dom = '<root>'

    default_mimetype = 'application/xml'

    def __init__(self, event=200, data={}, msg=''):
        self.__status = event
        self.__dom = self.__dom + '<event>\n' + str(event) + '\n</event>\n'
        self.__dom = self.__dom + '<data>\n' + self.parse_to_node(data) + '\n</data>\n'
        self.__dom = self.__dom + '<msg>\n' + str(msg) + '\n</msg>\n'
        self.__dom = self.__dom + '</root>'

    def parse_to_node(self, data):
        nodes = []
        if isinstance(data, dict):
            for k, v in data.items():
                nodes.append(
                    '<' + str(k) + '>\n' + (
                        str(v) if not isinstance(v, dict) else self.parse_to_node(v)) + '\n</' + str(
                        k) + '>\n')
        elif isinstance(data, list):
            for index in range(len(data)):
                nodes.append(
                    '<' + str(index) + '>\n' + (
                        str(data[index]) if not isinstance(data[index], dict) else self.parse_to_node(
                            data[index])) + '\n</' + str(
                        index) + '>\n')
        else:
            nodes.append(str(data))
        return '\n'.join(nodes)

    def as_res(self):
        return Response(str(self.__dom), mimetype=self.default_mimetype, status=self.__status)



from threading import Lock


class OutOfSizeError(IndexError):
    def __index__(self, msg: str = ''):
        super(OutOfSizeError, self).__index__(msg)


class DStack:

    def __init__(self, clz: type = object, max_size: int = -1):
        # 初始化相关配置
        self._clz = clz
        self._data = []
        self._lock = Lock()
        self._max = max_size

    def push(self, item: object):
        '''
        添加单元
        :param item:
        :return:
        '''
        assert isinstance(item, self._clz), TypeError(f'current item is not belong {self._clz}')
        # 栈最大值过滤断言
        assert self._max == -1 or len(self._data) <= self._max, OutOfSizeError('stack full of size')
        try:
            self._lock.acquire(False, 2)
        except:
            pass
        else:
            self._data.append(item)
        finally:
            self._lock.release()

    def pop(self, index: int = 0):
        '''
        弹出单元
        :param index:
        :return:
        '''
        assert len(self._data) > 0, IndexError('pop from empty stack')
        self._data.pop(index)

    def flush(self):
        '''
        冲刷数据域
        :return:
        '''
        self._data = []

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass



# coding: utf-8
from dophon.annotation import AutoWired
from dophon.annotation import response, request
from dophon.annotation.AutoWired import *
from dophon.annotation.description import *
from dophon.annotation.request import BaseHandler

"""
注解集合(部分)
author:CallMeE
date:2018-06-01


"""
__all__ = [
    'ResponseBody',
    'ResponseTemplate',
    'AutoParam',
    'FullParam',
    'RequestMapping',
    'GetRoute',
    'PostRoute',
    'Get',
    'Post',
    'AutoWired',
    'AsResBody',
    'AsResTemp',
    'AsArgs',
    'AsJson',
    'AsFile',
    'BeanConfig',
    'bean',
    'Bean',
    'Desc',
    'DefBean',
    'res',
    'req',
    'BaseHandler'
]

AutoWired = AutoWired

ResponseBody = AsResBody = response.response_body

ResponseTemplate = AsResTemp = response.response_template

AutoParam = AsArgs = request.auto_param

FullParam = AsJson = request.full_param

FileParam = AsFile = request.file_param

BeanConfig = AutoWired.BeanConfig

bean = AutoWired.bean

Bean = AutoWired.Bean

RequestMapping = request.request_mapping

GetRoute = request.get_route

PostRoute = request.post_route

Get = request.get

Post = request.post

Desc = Desc

DefBean = DefBean

res = response

req = request


# coding: utf-8
from dophon_properties import *
import os, sys
import xml.dom.minidom as dom

get_properties(DOPHON)

import properties
from dophon_logger import *

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())

name_sep = '_'

modules_list = []


def check_modules():
    """
    加载框架配置模块
    :return:
    """
    logger.info('校验模块')
    # logger.info('更新pip')
    # os.system(f'python -m pip install --upgrade pip')
    modules_path = properties.project_root + '/module.xml'
    if os.path.exists(modules_path):
        with open(modules_path) as modules_file:
            modules_tree = dom.parse(modules_file)
            # print(modules_tree)
            modules_info = modules_tree.getElementsByTagName('module')
            # print(modules_info)
            for module_info in modules_info:
                pre_name = module_info.getElementsByTagName('pre-name')
                pre_name = pre_name[0].childNodes[0].data if pre_name and pre_name[0].childNodes else None
                name = module_info.getElementsByTagName('name')
                name = name[0].childNodes[0].data if name and name[0].childNodes else None
                version = module_info.getElementsByTagName('version')
                version = version[0].childNodes[0].data if version and version[0].childNodes else None
                module_name = ((pre_name + name_sep) if pre_name else '') + name
                module_code_str = ((pre_name + '.') if pre_name else '') + name
                # 添加模块到模块列表
                modules_list.append(module_code_str)
                while True:
                    __module = None
                    try:
                        # 校验模块安装
                        module = __module if __module else __import__(module_name)
                        # 等待模块安装完成
                        sys.modules[module_code_str] = module
                        logger.info(f'模块{getattr(module,"__name__")}依赖建立完毕,引入别名{pre_name}.{name}')
                        break
                    except Exception as e:
                        # print(e)
                        logger.info(f"install {module_name} >={version if version else 'release'}")
                        # 利用pip模块安装所需模块
                        pip_arg_list = ['pip', 'install',
                                        module_name + (('>=' + version) if version else ''),
                                        '--user']
                        exe_pip_args = ['install',
                                        module_name + (('>=' + version) if version else ''),
                                        '--user']
                        if not version:
                            pip_arg_list.append('-U')
                            exe_pip_args.append('-U')
                        # 利用pip安装模块
                        import pip
                        from pip._internal import main as _main
                        # path = os.path.dirname(os.path.dirname(__file__))
                        # sys.path.insert(0, path)
                        os.system(f'pip install {module_name}')
                        # _main(['list'])
                        # _main(exe_pip_args)
                        __module = __import__(module_name)
                        # raise ModuleNotFoundError(
                        #     f"please use '{' '.join(pip_arg_list)}' to install module %s ")
    logger.info('模块校验完毕')



# coding: utf-8
from multiprocessing import Process, freeze_support
from concurrent.futures import ProcessPoolExecutor
from threading import Thread
import time, socket, random
from flask import request, make_response
from urllib3 import PoolManager
from dophon import properties
from dophon_logger import *
from dophon.annotation import Bean
import requests as direct_r
from requests_toolbelt.multipart.encoder import MultipartEncoder
from dophon.boot import BeanScan

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())

ports = []  # 记录监听端口

proxy_clusters = {}

pool = PoolManager()


def main_freeze():
    freeze_support()


def redirect_request():
    current_header = {}
    request_kwargs = {}
    form_boundary = None
    # 迁移当前请求头
    for k, v in request.headers.items():
        if v.startswith('multipart/form-data;'):
            form_header_info = v.split('boundary=')
            form_boundary = form_header_info[1] if len(form_header_info) > 1 else None
            continue
        current_header[k] = v
    # print(request.endpoint)
    if request.endpoint and not request.endpoint.startswith('_annotation_auto_reg'):
        pass
    elif not current_header.get('Outer'):
        choice_port = random.choice(ports)
        # 添加网关标识
        current_header['Outer'] = 'pass'
        logger.info(f'touch path: {request.path}, touch port: {choice_port} [success]')
        request_kwargs['headers'] = current_header
        request_fields = request.json if request.is_json else request.form
        if form_boundary:
            __file = request.files.to_dict()['file']
            request_kwargs['data'] = {k: v for k, v in request_fields.items()}
            request_kwargs['data']['file'] = (__file.filename, __file.stream, __file.content_type)
            request_kwargs['data'] = MultipartEncoder(fields=request_kwargs['data'])
            request_kwargs['headers']["Content-Type"] = request_kwargs['data'].content_type
            res = direct_r.request(
                request.method,
                f'http://127.0.0.1:{choice_port}{request.path}',
                **request_kwargs
            )
            cell_res = make_response(res.content, res.status_code)
        else:
            if request.content_type and request.content_type.startswith('application/json'):
                import json
                request_kwargs['data'] = request_fields
            else:
                request_kwargs['data'] = request_fields

            # res = pool.request(
            #     request.method,
            #     f'http://127.0.0.1:{choice_port}{request.path}',
            #     **request_kwargs
            # )
            res = direct_r.request(
                request.method,
                f'http://127.0.0.1:{choice_port}{request.path}',
                **request_kwargs
            )
            cell_res = make_response(res.content, res.status_code)
        # print(res)
        # 迁移转发的响应头
        for item in res.headers:
            # print(item)
            cell_res.headers[item] = res.headers[item]
        return cell_res


def outer_entity(boot):
    # 重写路由信息(修改为重定向路径)
    Bean('app').before_request(redirect_request)
    boot.d_web(host='0.0.0.0')(lambda x: x)(0)


process_pool = ProcessPoolExecutor()


def run_clusters(base_boot, clusters: int, outer_port: bool = False, start_port: int = 8800,
                 multi_static_fix: bool = False,
                 part_kwargs: dict = {}):
    """
    运行集群式服务器
    :param clusters: 集群个数
    :param outer_port: 是否开启外部端口映射(映射端口为用户配置文件中配置的端口)
    :param start_port: 集群起始监听端口
    :param multi_static_fix: 集群静态文件修复
    :param part_kwargs: 集群节点额外参数(会覆盖默认参数)
    :return:
    """
    from dophon import boot, webboot
    process_pool = ProcessPoolExecutor(max_workers=clusters)
    if base_boot.__name__ == webboot.__name__:
        for i in range(clusters):
            current_port = start_port + i
            create_cluster_web_cell(boot=base_boot, port=current_port, part_kwargs=part_kwargs,
                                    cell_static_fix=multi_static_fix)
            ports.append(current_port)
        while len(ports) != clusters:
            time.sleep(5)

        logger.info('启动检测端口监听')
        for port in ports:
            if check_socket(int(port)):
                continue
        logger.info('集群端口: %s ' % ports)
        if outer_port:
            logger.info('启动外部端口监听[%s]' % (properties.port))
            outer_entity(base_boot)


def action_webboot(boot, cell_static_fix: bool, port: int):
    try:
        boot.fix_static(enhance_power=cell_static_fix)
        boot.fix_template()
    except Exception as e:
        raise e
    return boot.run


def create_cluster_web_cell(boot, port, part_kwargs, cell_static_fix: bool):
    kwargs = {
        'host': '127.0.0.1',
        'port': port,
        'run_type': eval(f'boot.{part_kwargs["run_type"] if "run_type" in part_kwargs else "FLASK"}'),
        'n': 2
    }
    if part_kwargs:
        # 迁移参数
        for k, v in part_kwargs.items():
            if k not in kwargs:
                kwargs[k] = v
    cell_boot = action_webboot(boot, cell_static_fix, port)
    # print(cell_boot)
    # http协议
    from dophon.tools import is_not_windows
    # Linux使用多进程
    # Windows使用多线程
    proc = Process(target=cell_boot, kwargs=kwargs) if is_not_windows() else Thread(target=cell_boot, kwargs=kwargs)
    port_proc = Process(target=check_socket, args=(int(port),)) if is_not_windows() else Thread(target=check_socket,
                                                                                                args=(int(port),))
    proc.start()
    port_proc.start()
    port_proc.join()
    # process_pool.submit(cell_boot, **kwargs)


def check_socket(port: int):
    s = socket.socket()
    flag = True
    while flag:
        try:
            ex_code = s.connect_ex(('127.0.0.1', port))
            flag = False
            return int(ex_code) == 0
        except Exception as e:
            time.sleep(3)
            continue



import re
from json import JSONEncoder


def to_dict(obj: object) -> dict:
    result = {}
    for name in dir(obj):
        if re.match('__.*__', name):
            continue
        elif callable(getattr(obj, name)):
            continue
        else:
            result[name] = getattr(obj, name)
    return result


def to_lower_camel(name: str):
    result = []
    str_len = len(name)
    for c_index in range(str_len):
        c = name[c_index]
        if c.islower():
            result.append(c)
            continue
        if (c_index + 1) < str_len and name[c_index + 1].islower():
            __append = c.lower()
        else:
            __append = c
        if c_index <= 0:
            result.append(__append)
            continue
        result.append(f'_{__append}' if __append.islower() else f'{__append}')
    return ''.join(result)


class ObjectEncoder(JSONEncoder):

    def default(self, o):
        if isinstance(o, object):
            return to_dict(o)
        return JSONEncoder.default(self, o)



from .boot import load_banner, load_footer
from dophon import properties, blue_print
from dophon.boot import d_app
import traceback
from datetime import datetime
import json
import re
import sys
from threading import *
from flask import Flask, request, abort, jsonify
import os
from html import escape
from dophon.tools import gc
from .tools.dynamic_import import d_import
import functools
from . import tools
from .tools.sys_utils import ObjectEncoder

from dophon_logger import *

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())

from .tools.framework_const.objs import DStack

# 旧版本与数据库的耦合,后期版本移除
try:
    mysql = __import__('dophon.db.mysql')
except:
    mysql = None

rps_stack = DStack(int, properties.rps_max) if properties.rps else None

if properties.rps:
    Timer(1, rps_stack.flush).start()

# 初始化错误信息
error_info = properties.error_info


def web_boot_init(mode: int = 1):
    """
    初始化启动
    :return:
    """
    assert mode > 0, InterruptedError('非法模式')

    def init_web_boot():
        if mode == 1:
            global app_name, app, ip_count, ipcount_lock, ip_refuse_list
        else:
            global multi_apps
            multi_apps = {}
        app_name = properties.service_name if hasattr(properties, 'service_name') else __name__
        # 定义WEB容器(同时防止json以ascii解码返回)
        app = Flask(app_name)
        from .annotation.AutoWired import obj_manager
        if mode == 1:
            obj_manager.reg_bean('app', app)
        else:
            obj_manager.reg_bean(f'__{id(app)}app', app)
            multi_apps[id(app)] = app
        app.config['JSON_AS_ASCII'] = False

        # ip计数缓存
        ip_count = {}
        ipcount_lock = Lock()
        if os.path.exists(os.getcwd() + '/ip_count'):
            with open('ip_count', 'r') as ip_count_file:
                file_text = ip_count_file.read()
                if file_text:
                    ip_count = eval(file_text)
        else:
            with open('ip_count', 'w') as ip_count_file:
                json.dump({}, ip_count_file)
        # IP黑名单
        ip_refuse_list = {}
        return app

    return init_web_boot()


web_boot_init()


def before_request():
    """
    定义拦截器
    :return:
    """
    if properties.ip_count:
        # 开启ip技术计数
        ipcount_lock.locked()
        ip = request.remote_addr
        now_timestemp = datetime.now().timestamp()
        # 检测是否为黑名单
        if ip in ip_refuse_list:
            # 默认禁用一分钟
            if (int(now_timestemp) - int(ip_refuse_list[ip])) > 60:
                # 可以清除白名单
                ip_refuse_list.pop(ip)
                # 清除访问记录
                ip_count[ip]['req_timestemp'] = [now_timestemp]
            else:
                # 未到解禁时间
                return abort(400)
        if ip in ip_count:
            ip_item = ip_count[ip]
            ip_item['req_count'] += 1
            req_timestemp = ip_item['req_timestemp']
            """
            判断逻辑:
                当前请求时间是最近的,连续的1秒内
            """
            if (int(now_timestemp) - int(req_timestemp[0])) > 1 \
                    and \
                    (int(now_timestemp) - int(req_timestemp[len(req_timestemp) - 1])) < 1:
                # 检测3秒内请求数
                if len(req_timestemp) > 50:
                    # 默认三秒内请求不超过300
                    # 超出策略则添加到黑名单
                    ip_refuse_list[ip] = now_timestemp
                else:
                    # 不超出策略则清空请求时间戳缓存
                    ip_item['req_timestemp'] = [now_timestemp]
            else:
                ip_item['req_timestemp'].append(now_timestemp)
        else:
            ip_item = {
                'req_count': 1,
                'req_timestemp': [now_timestemp]
            }
        Thread(target=persist_ip_count).start()
        ip_count[ip] = ip_item

    if properties.rps:
        # 开启rps监控
        # 使用漏桶算法
        global rps_stack
        rps_stack.push(1)


def persist_ip_count():
    """
    持久化ip统计到文件
    :return:
    """
    # 持久化ip统计
    with open('ip_count', 'w') as ip_count_file:
        json.dump(ip_count, ip_count_file)
    if ipcount_lock.locked():
        ipcount_lock.release()


blueprint_init_queue = {}  # 蓝图初始化方法缓存(用于初始化后启动)


# 处理各模块中的自动注入以及组装各路由
# dir_path中为路由模块路径,例如需要引入的路由都在routes文件夹中,则传入参数'/routes'
def map_apps(dir_path):
    path = os.getcwd() + dir_path
    if not os.path.exists(path):
        logger.error('路由文件夹不存在,创建路由文件夹')
        os.mkdir(path)
    f_list = os.listdir(path)
    logger.info(f'路由文件夹: {dir_path}')
    while f_list:
        try:
            file = f_list.pop(0)
            if re.match('__.*__', file):
                continue
            i = os.path.join(path, file)
            if os.path.isdir(i):
                logger.info(f'加载路由模块: {file}')
                # 扫描内部文件夹
                map_apps(f'{dir_path}{os.sep}{file}')
                continue
            file_name = re.sub('\.py', '', file)
            module_path = re.sub('^\.', '', re.sub('\\\\|/', '.', dir_path))
            # print(module_path)
            f_model = __import__(f'{module_path}.{file_name}', fromlist=True)

            # 自动装配蓝图实例并自动配置部分参数,免除繁琐配置以及精简代码
            package_app = getattr(f_model, '__app') \
                if hasattr(f_model, '__app') \
                else f_model.app \
                if hasattr(f_model, 'app') \
                else blue_print(f"_boot_auto_reg.{file_name}", getattr(f_model, '__name__'))
            package_app.url_prefix = getattr(f_model, 'PREFIX', '')
            filter_method = package_app.before_request(before_request)
            # 若需统计请求,装配请求统计方法
            if hasattr(properties, 'ip_count') and getattr(properties, 'ip_count'):
                setattr(f_model, 'before_request', filter_method)
            # 若存在初始化执行方法,执行该方法
            if hasattr(f_model, 'blueprint_init'):
                init_fun = getattr(f_model, 'blueprint_init')
                # 判断是否方法
                if callable(init_fun):
                    blueprint_init_queue[f_model] = before_bp_init_fun(init_fun)
            get_app().register_blueprint(package_app)

        except Exception as e:
            raise e
            pass

            # for name in dir(item):
            #     print(f'{name} ==> {getattr(item, name)}')

            # print(get_app().blueprints)


def before_bp_init_fun(f):
    """
    预留蓝图初始化装饰器
    :param f:
    :return:
    """

    # print(f)

    def fields(*args, **kwargs):
        # print('args: ', args, 'kwargs:', kwargs)
        f(*args, **kwargs)

    return fields


def get_app() -> Flask:
    return app


@d_app
def free_source():
    def method(f):
        @functools.wraps(f)
        def args(*arg, **kwarg):
            try:
                logger.info('启动服务器')
                logger.info('路由初始化')
                logger.debug('mapping routes')
                for path in properties.blueprint_path:
                    map_apps(path)
                # 执行蓝图初始化方法
                for blueprint_module, blue_print_init_method in blueprint_init_queue.items():
                    try:
                        blue_print_init_method()
                    except Exception as e:
                        logger.error(f'蓝图"{blueprint_module}"初始化失败,信息: {e}')
                # for rule in get_app().url_map.iter_rules():
                #     print(str(rule))
                # print(rule.get_rules())
                # for name in dir(rule):
                #     print(f'{name}---{getattr(rule,name)}')
                # print(help(rule))
                # break
                f(*arg, **kwarg)
                """
                释放所有资源
                :return:
                """
                logger.info('服务器关闭')
                logger.debug('释放资源')
                if mysql:
                    mysql.free_pool()
                logger.debug('释放连接池')
                sys.exit()
                # logger.info('再次按下Ctrl+C退出')
            except Exception as e:
                logger.error(e)

        return args

    return method


def fix_static(
        fix_target: Flask = app,
        static_floder: str = 'static',
        enhance_power: bool = False
):
    """
    修正静态文件路径
    :return:
    """
    if hasattr(fix_target, 'static_folder'):
        if hasattr(properties, 'project_root'):
            root_path = properties.project_root
        else:
            root_path = os.getcwd()
        static_floder_path = root_path + '/' + static_floder
        fix_target.static_folder = static_floder_path
        if os.path.exists(static_floder_path) and enhance_power:
            enhance_static_route(static_floder_path)
    else:
        raise Exception('错误的修复对象')


def enhance_static_route(static_floder_path: str):
    """
    增强静态文件路由能力
    :param static_floder_path:
    :param serlize:
    :return:
    """
    framework_static_route_path = f'{properties.project_root}{properties.blueprint_path[0]}/FrameworkStaticRoute.py'
    # if not os.path.exists(framework_static_route_path):
    linux_static_floder_path = static_floder_path.replace('\\', '/')
    # 防止多次静态文件路由,只注册一次
    framework_static_route = 'FrameworkStaticRoute'
    framework_static_route_lock = False
    for r in get_app().blueprints:
        if re.search(r, framework_static_route):
            # 路由已注册
            framework_static_route_lock = True
            break
    if not framework_static_route_lock:
        # import types
        import uuid
        # 定义静态资源获取路径
        logger.info('增强静态文件路由')
        with open(framework_static_route_path, 'wb') as fsroute:
            # 写入预设信息
            fsroute.write(bytes(
                f"# -*- coding: utf-8 -*-\n"
                f"from dophon import blue_print\n"
                f"from dophon.annotation import *\n"
                f"from flask import url_for\n"
                f"from flask import render_template\n"
                f"from flask import send_from_directory\n"
                f"app = blue_print('{framework_static_route}', __name__,static_folder='{linux_static_floder_path}')\n",
                encoding='utf-8'))
            for root, dir_path, file in os.walk(linux_static_floder_path):
                # 消除转义字符(win)
                root = re.sub('\\\\', '/', root)
                linux_static_floder_path = re.sub('\\\\', '/', linux_static_floder_path)
                # 静态目录下的目录名
                root_dir_path = re.sub('\\\\', '/', re.sub(linux_static_floder_path, "", root))
                # 解析静态资源路径
                route_name = re.sub('[^(1-9a-zA-Z_)]', '',
                                    f'{re.sub(linux_static_floder_path, "", root)}_{root_dir_path}_{uuid.uuid1()}')
                route_path = f'{root_dir_path}/<file_name>'
                static_url_method_code_body = \
                    f"@RequestMapping('{route_path}',['get','post'])\ndef {route_name}(file_name):\n\t" \
                        f"\"\"\"\n\t" \
                        f"static file route in {route_path}\n\t" \
                        f":param file_name: static file name\n\t" \
                        f":return static file blob\n\t" \
                        f"\"\"\"\n\t" \
                        f"return render_template(f'{root_dir_path}/" \
                        "{file_name}" \
                        f"') if file_name.endswith('.html') " \
                        f"else send_from_directory('{linux_static_floder_path}{root_dir_path}',f" \
                        "'{file_name}'" \
                        ",as_attachment=True)\n"
                fsroute.write(bytes(
                    static_url_method_code_body,
                    encoding='utf-8'))
        logger.info("增强静态文件夹完毕")


def fix_template(
        fix_target: Flask = app,
        template_folder: str = 'templates'
):
    """
    修正页面模板文件路径
    :return:
    """
    if hasattr(fix_target, 'template_folder'):
        if hasattr(properties, 'project_root'):
            root_path = properties.project_root
        else:
            root_path = os.getcwd()
        fix_target.template_folder = root_path + '/' + template_folder
    else:
        raise Exception('错误的修复对象')


@free_source()
def run_app(host=properties.host, port=properties.port, __app=app):
    logger.info(f'监听地址: {host} : {port}')
    if properties.server_threaded:
        # 开启多线程处理
        __app.run(host=host, port=port, threaded=properties.server_threaded)
    elif tools.is_not_windows() and properties.server_processes > 1:
        # 开启多进程处理
        print('开启多进程', properties.server_processes)
        __app.run(host=host, port=port, threaded=False, processes=properties.server_processes)
    else:
        __app.run(host=host, port=port)


@free_source()
def tornado(host=properties.host, port=properties.port):
    logger.info(f'添加Tornado核心')
    d_import('tornado')
    from tornado.wsgi import WSGIContainer
    from tornado.httpserver import HTTPServer
    from tornado.ioloop import IOLoop
    logger.info(f'开始使用Tornado增强')
    http_server = HTTPServer(WSGIContainer(get_app()))
    logger.info(f'监听{port}')
    if tools.is_not_windows() and properties.server_processes > 1:
        http_server.bind(port, host)  # flask默认的端口
        http_server.start(properties.server_processes)
    else:
        http_server.listen(port, host)  # flask默认的端口
    logger.info(f'监听地址: {host}:{port}')
    IOLoop.current().start()


@free_source()
def run_app_ssl(host=properties.host, port=properties.port, ssl_context=properties.ssl_context):
    logger.info(f'监听地址: {host} : {port}')
    if properties.server_threaded:
        # 开启多线程处理
        app.run(host=host, port=port, ssl_context=ssl_context, threaded=properties.server_threaded)
    elif tools.is_not_windows() and properties.server_processes > 1:
        # 开启多进程处理
        app.run(host=host, port=port, ssl_context=ssl_context, threaded=False,
                processes=properties.server_processes)
    else:
        app.run(host=host, port=port, ssl_context=ssl_context)


@free_source()
def tornado_ssl(host=properties.host, port=properties.port, ssl_context=properties.ssl_context):
    logger.info(f'添加Tornado核心')
    from tornado.wsgi import WSGIContainer
    from tornado.httpserver import HTTPServer
    from tornado.ioloop import IOLoop
    logger.info(f'开始使用Tornado增强')
    http_server = HTTPServer(WSGIContainer(get_app()), ssl_options={
        'certfile': ssl_context[0],
        'keyfile': ssl_context[1],
    })
    logger.info(f'监听{port}')
    if tools.is_not_windows() and properties.server_processes > 1:
        http_server.bind(port)  # flask默认的端口
        http_server.start(properties.server_processes)
    else:
        http_server.listen(port)  # flask默认的端口
    logger.info(f'监听地址: {host}:{port}')
    IOLoop.current().start()


def run_f(ssl: bool = False, **kwargs):
    """
    flask 启动入口
    :param ssl:
    :param kwargs:
    :return:
    """
    run_app_ssl(**kwargs) if ssl else run_app(**kwargs)


def run_t(ssl: bool = False, **kwargs):
    """
    tornado 启动入口
    :param ssl:
    :param kwargs:
    :return:
    """
    tornado_ssl(**kwargs) if ssl else tornado(**kwargs)


FLASK = 0
TORNADO = 1


def run(run_type: int = FLASK, ssl: bool = False, n: int = 1, **kwargs):
    """
    统一入口
    :param run_type: 启动类型
    :param ssl:  https开关
    :param kwargs: 启动参数
    :return:
    """
    if n > 1:
        kwargs['__app'] = web_boot_init(2)
    try:
        run_f(ssl, **kwargs) if run_type == FLASK else run_t(ssl, **kwargs) if run_type == TORNADO else 1 / 0
    except ZeroDivisionError:
        from dophon.errors.ParamErrors import BootParamError
        raise BootParamError('web container boot fail,please check your boot param in <function run>')


def d_web(run_type: int = FLASK, ssl: bool = False, **options):
    """
    装饰器形式web容器启动
    :param run_type: 启动类型
    :param ssl:  https开关
    :param options: 启动参数
    :return:
    """

    def _d_web(f):
        @functools.wraps(f)
        def __d_web(*args, **kwargs):
            f(*args, **kwargs)
            return run(run_type, ssl, **options)

        return __d_web

    return _d_web


def bootstrap_app():
    """
    bootstrap样式页面初始化
    :return:
    """
    global app
    b = __import__('flask_bootstrap')
    b.Bootstrap(app)


# region 定义内部接口

from dophon.annotation import *


@GetRoute('/find/function/desc/by/module/<module_path>')
@ResponseBody()
def find_function_desc_by_module(module_path: str):
    print(module_path)


@RequestMapping('/framework/ip/count', ['get', 'post'])
@ResponseBody()
def view_ip_count():
    """
    此为公共请求,不计入ip统计
    :return:
    """
    return ip_count


@RequestMapping('/framework/ip/refuse', ['get', 'post'])
@ResponseBody()
def view_ip_refuse():
    """
    此为公共请求,不计入ip统计
    :return:
    """
    return ip_refuse_list


GC_INFO = properties.debug_trace if properties.debug_trace else False  # gc信息开关


@RequestMapping('/framework/gc/show', ['get'])
@ResponseBody()
def show_gc_info():
    # return gc.show_gc_leak(print)
    return gc.show_gc_leak(logger.info) if GC_INFO else {}


@GetRoute('/rule/<view>')
def show_rule_info(view):
    if not properties.load_desc:
        abort(404)
    __bind_rule_info_map = {}

    for item in get_app().url_map.iter_rules():
        # print(next(get_app().url_map.iter_rules(item.endpoint)))
        # print(re.sub('_annotation_auto_reg\.', '', item.endpoint))

        __bind_rule_info_map[str(next(get_app().url_map.iter_rules(item.endpoint)))] = \
            {
                'end_point': re.sub('_annotation_auto_reg\.', '', item.endpoint),
                'methods': item.methods
            }

    # print([ v for v in sorted(__bind_rule_info_map.values())] )

    from .annotation.description.describe import DESC_INFO
    __DESC_INFO = DESC_INFO
    # print(DESC_INFO)
    __json_viewer_scripts = []
    try:
        def __sort_inner_desc_info(item_key: str, item_id: int):
            def parse_param(info: dict):
                __result = ''
                __result += '<div class="col p-3">'
                __result += f'''<div style="margin: 10px;display:flex;" class="">
                            <span class="label label-primary h6" style="margin-right:auto;">
                                Request Params
                            </span>
                            </div>'''
                __result += '<div class="p-3 d-flex">' \
                            '<table class="table table-hover table-sm table-striped w-100">' + \
                            '''<thead>
                                <th>name</th>
                                <th>default_value</th>
                                <th>desc</th>
                                <th>type</th>
                            </thead>''' + \
                            '<tbody>'
                for __param_name, __param_info in info.items():
                    __result += f'''<tr>'''
                    for __name, __value in __param_info.items():
                        __result += f'''
                            <td class="">{escape(str(__value)) if __value != str(None) else ''}</td>
                             '''
                    __result += '</tr>'
                __result += '</tbody></table>'
                __result += '</div></div>'
                return __result

            result = ''
            if __bind_rule_info_map[item_key]['end_point'] in __DESC_INFO:
                __desc_info = __DESC_INFO[__bind_rule_info_map[item_key]['end_point']]
            else:
                return result
            for __arg_name, __arg_info in __desc_info.items():
                if __arg_name == 'demo':
                    if __arg_info:
                        # 添加参数demo
                        __json_viewer_scripts.append(f"""
                            $("#param_demo_{id(item_key)}_{id(__arg_name)}")""" + \
                                                     f""".html(formatJson({json.dumps(__arg_info,
                                                                                      ensure_ascii=False,
                                                                                      cls=ObjectEncoder)}));
                        """)
                    result += f"""
                    <div class="row row-cols-2 mb-3 mt-5">
                        <div class="col">
                            <div class="">
                                <h4 class="">
                                    Param Demo:
                                </h4>
                            </div>
                            <div id="collapse_{id(__arg_name)}_{id(__arg_info)}">
                                <pre class="pre-scrollable w-100 h-100" id="param_demo_{id(item_key)}_{id(
                        __arg_name)}"></pre>
                            </div>
                        </div>
                        """
                elif __arg_name == 'own_doc':
                    result += f"""
                            <div class="w-50">
                                <div class="">
                                    <h4 class="">
                                        Doc:
                                    </h4>
                                </div>
                                <div id="collapse_{id(__arg_name)}_{id(__arg_info)}">
                                    <div class="h-100 {'h6 pl-2'}">
                                        { __arg_info}
                                    </div>
                                </div>
                            </div>
                        """
                elif __arg_name == 'args_info':
                    result += f"""
                            <div class="w-100">
                                <div class="">
                                    <h4 class="">
                                        <br/>Params:
                                    </h4>
                                </div>
                                <div id="collapse_{id(__arg_name)}_{id(__arg_info)}">
                                    <div class="{'row'} d-flex">
                                        {parse_param(__arg_info) }
                                    </div>
                                </div>
                            </div>
                        """
                elif __arg_name == 'returns':
                    # 添加返回值demo
                    if __arg_info['type'] != 'any':
                        __json_viewer_scripts.append(f"""
                            $("#func_returns_demo_{id(__arg_name)}_{id(__arg_info)}")""" + \
                                                     f""".html(formatJson({json.dumps(__arg_info['demo'],
                                                                                      ensure_ascii=False,
                                                                                      cls=ObjectEncoder)}));
                        """)
                    result += f"""
                        <div class="col h-100">
                                <div class="">
                                    <div class="h4 d-flex">
                                        <span class="mr-auto">Return</span> 
                                        <span class="badge ml-auto">{__arg_info['type']}</span>
                                    </div>
                                </div>
                                <div id="func_returns_{id(__arg_name)}_{id(__arg_info)}">
                                    """ + (f"""
                                        <pre id="func_returns_demo_{id(__arg_name)}_{id(
                        __arg_info)}" class=" w-100 h-100 pre-scrollable"></pre>
                                    """ if __arg_info['type'] != 'any' else "") + """
                                </div>
                        </div>
                    </div>
                        """
                elif __arg_name == 'remark' and __arg_info:
                    # 装载接口描述
                    __json_viewer_scripts.append(f"""
                        $("#function_remark_{item_id}").html(
                        $("#function_remark_{item_id}").html() + 
                        "<span class='badge badge-Secondary h5 ml-auto mt-auto mb-auto'>{__arg_info}</span>" + 
                        "<div class='h-100 ml-1 mr-1 mt-0 mb-0 border-right'>&nbsp;</div>&nbsp;"
                        );
                    """)
                else:
                    continue

            return result

        # 注册路径列表入口

        #    {'</div></div><div class="ivu-card ivu-card-bordered"><div class="ivu-card-body">'.join(
        #     [
        #         str(item)
        #         for item in get_app().url_map.iter_rules()
        #
        #     ]
        # )}
        body_str = ''
        METHODS_CLASS = {
            'GET': 'badge-success',
            'POST': 'badge-warning',
        }
        __blueprint_group = ''
        __app_rules = list(get_app().url_map.iter_rules())
        __app_rules.sort(
            key=lambda i: (str(i)
                           if re.search('\.', str(i.endpoint)) and not re.match('FrameworkStaticRoute\..*',
                                                                                str(i.endpoint))
                           else 'FrameworkStaticRoute'))
        __current_buleprint_group = None
        __blueprint_group_split = False
        for item in __app_rules:
            route_method_path = __bind_rule_info_map[str(item)]['end_point']
            route_method_methods = __bind_rule_info_map[str(item)]['methods']
            __current_buleprint_module = '.'.join(route_method_path.split('.')[:-1])
            v_methods_tag = [f"""
                    <span class="badge badge-pill {METHODS_CLASS[m]}">
                        {m}
                    </span>
                """ for m in route_method_methods - {'OPTIONS', 'HEAD'}]
            if re.search('\.', route_method_path) and not re.match('FrameworkStaticRoute\..*', route_method_path):
                # 路由接口分组
                if __current_buleprint_group != __current_buleprint_module:
                    if __current_buleprint_group:
                        body_str += '</div>'
                    __current_buleprint_group = __current_buleprint_module
                    body_str += f'''<div class="pl-3 pr-3 h5 pb-1">
                    <div class="d-flex">
                    <div class="border border-secondary w-100 m-auto"></div>
                    <div class="ml-4 mr-4">{__current_buleprint_module}</div>
                    <div class="border border-secondary m-auto" style="width:5%;"></div>
                    </div>'''
                # 非框架定义的路由参与路由显示
                body_str += f"""
                    <div class="card mb-1">
                        <div class="card-header border-bottom bg-light p-1" style="padding:0px;display:flex;">
                            <h6 class="card-title" style="padding:5px;margin:0px;width:100%;">
                                <div data-toggle="collapse"  
                                    style="cursor:pointer;width:100%;display:flex;"
                                    data-target="#collapse_{id(item)}"
                                    aria-expanded="false" 
                                    aria-controls="collapse_{id(item)}">
                                    <span style="padding:2px;float:left;">
                                        {''.join(v_methods_tag)}
                                    </span>
                                    <span style="margin:auto auto auto 5;" class="d-flex w-100 h-100" id="function_remark_{id(
                    item)}">
                                        {escape(str(item))}
                                    </span>
                                    <span class="badge badge-info" style="float:right;margin:auto 0 auto auto;">
                                        {route_method_path.split('.')[-1]}
                                    </span>
                                </div>
                            </h6>
                        </div>
                        <div id="collapse_{id(item)}" class="collapse card-body">
                            <div class="h6 pt-1 pb-1">
                                <div class="h4">Api:</div>{escape(str(item))}
                            </div>
                            <div class="">
                                { __sort_inner_desc_info(str(item), id(item)) }
                            </div>
                        </div>
                    </div>"""
                if __blueprint_group_split and __current_buleprint_group != __current_buleprint_module:
                    body_str += '</div>'
                if __current_buleprint_group:
                    __blueprint_group_split = True
            else:
                __blueprint_group_split = False
                body_str += '</div>'
        from dophon.web_templates import javascript, css
        page_data = f"""
                <html>
                <head>
                    <meta charset="utf-8"> 
                    <title>infos</title>
                    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap@4.4.1/dist/css/bootstrap.min.css" integrity="sha384-Vkoo8x4CGsO3+Hhxv8T/Q5PaXtkKtu6ug5TOeNV6gBiFeWPGFN9MuhOf23Q9Ifjh" crossorigin="anonymous" />
                    <style>
                        {css.JSONViewer}
                    </style>
                    <script type="text/javascript" src="https://code.jquery.com/jquery-3.4.1.min.js" ></script>
                    <script type="text/javascript">
                        {javascript.JSONViewer}
                    </script>

                </head>
                <body>
                    <div class="jumbotron jumbotron-fluid">
                      <div class="container">
                        <h1 class="display-4">{properties.app_name if hasattr(properties,
                                                                              'app_name') else 'Dophon Web'}</h1>
                        <p class="lead">Dophon Web Apis</p>
                      </div>
                    </div>
                    <div style="padding:10px;">
                       <div class="list-group" id="accordion">
                       {body_str}
                       </div>
                    </div>
                    <script src="https://cdn.jsdelivr.net/npm/jquery@3.4.1/dist/jquery.slim.min.js" integrity="sha384-J6qa4849blE2+poT4WnyKhv5vZF5SrPo0iEjwBvKU7imGFAV0wwj1yYfoRSJoZ+n" crossorigin="anonymous"></script>
                    <script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.0/dist/umd/popper.min.js" integrity="sha384-Q6E9RHvbIyZFJoft+2mJbHaEWldlvI9IOYy5n3zV9zzTtmI3UksdQRVvoxMfooAo" crossorigin="anonymous"></script>
                    <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.4.1/dist/js/bootstrap.min.js" integrity="sha384-wfSDF2E50Y2D1uUdj0O3uMBJnjuUD4Ih7YwaYd1iqfktj0Uod8GCExl3Og8ifwB6" crossorigin="anonymous"></script>
                    <script>
                        """"""
                        $(function(){
                        """f"""
                        {''.join(__json_viewer_scripts)}
                        """"""
                        })
                        """"""
                    </script>
                </body>
                </html>
           """ if view == 'map' else jsonify([
            str(item)
            for item in get_app().url_map.iter_rules()

        ]) if view == 'json' else abort(404)
        return page_data
    except AssertionError:
        pass


# endregion

# region 定义报错页面

from dophon.tools.framework_const import error_info_type, self_result


@app.errorhandler(500)
def handle_500(e):
    """
    处理业务代码异常页面
    :param e:
    :return:
    """
    exc_type, exc_value, exc_tb = sys.exc_info()
    tc = traceback.format_tb(exc_tb)
    if properties.debug_trace:
        trace_detail = '<h4>Details</h4>'
        trace_detail += '<table style="width:100%;" cellspacing="0" cellpadding="4">'
        for tc_item in tc:
            if tc_item.find(os.getcwd()) < 0:
                continue
            trace_detail += '<tr>'
            tc_item_info_list = tc_item.split(',', 2)
            # for tc_item_info_list_item in tc_item_info_list:
            #     trace_detail += '<td>' + tc_item_info_list_item + '</td>'
            trace_detail += '<td style="border: 1px solid gray;">' + re.sub('(^.+(\\\|/))|"', '',
                                                                            tc_item_info_list[0]) + '</td>'
            trace_detail += '<td style="border: 1px solid gray;">' + tc_item_info_list[1] + '</td>'
            trace_code_detail = re.sub('^\s+', '', tc_item_info_list[2]).split(' ', 1)
            trace_detail += '<td style="border: 1px solid gray;">' + ' => '.join([
                trace_code_detail[0],
                re.sub('\s+', ' => ', trace_code_detail[1], 1),
            ]) + '</td>'
            trace_detail += '<tr>'
        trace_detail += '</table>'
    else:
        trace_detail = ''
    if error_info == error_info_type.HTML:
        return '<h1>Wrong!!</h1>' + \
               '<h2>error info:' + str(e) + '</h2>' + \
               '<h3>please contact coder or direct to ' \
               '<a href="https://dophon.blog">dophon</a> and leave your question</h3>' + \
               trace_detail, 500
    elif error_info == error_info_type.JSON:
        return self_result.JsonResult(500, tc, """
        please contact coder or direct to dophon website and leave your question
        """).as_res()
    elif error_info == error_info_type.XML:
        return self_result.XmlResult(500, tc, """
        please contact coder or direct to dophon website and leave your question
        """).as_res()


@app.errorhandler(404)
def handle_404(e):
    """
    处理路径匹配异常
    :return:
    """
    global error_info
    if error_info == error_info_type.HTML:
        return '<h1>Wrong!!</h1>' + \
               '<h2>error info:' + str(e) + '</h2>' + \
               '<h3>please contact coder or direct to ' \
               '<a href="https://dophon.blog">dophon</a> and leave your question</h3>' + \
               'request path:' + request.path, 404
    elif error_info == error_info_type.JSON:
        return self_result.JsonResult(404, request.path, 'please check your path').as_res()
    elif error_info == error_info_type.XML:
        return self_result.XmlResult(404, request.path, 'please check your path').as_res()


@app.errorhandler(405)
def handle_405(e):
    """
    处理请求方法异常
    :return:
    """
    if error_info == error_info_type.HTML:
        return '<h1>Wrong!!</h1>' + \
               '<h2>error info:' + str(e) + '</h2>' + \
               '<h3>please contact coder or direct to ' \
               '<a href="https://dophon.blog">dophon</a> and leave your question</h3>' + \
               'request method:' + request.method, 405
    elif error_info == error_info_type.JSON:
        return self_result.JsonResult(405, {request.path, request.method}, 'please check your request method').as_res()
    elif error_info == error_info_type.XML:
        return self_result.XmlResult(405, {request.path, request.method}, 'please check your request method').as_res()


@app.errorhandler(400)
def handle_400(e):
    """
    处理异常请求
    :return:
    """
    if error_info == error_info_type.HTML:
        return ('<h1>Wrong!!</h1>' + \
                '<h2>error info:' + str(e) + '</h2>' + \
                '<h3>please contact coder or direct to '
                '<a href="https://dophon.blog">dophon</a> and leave your question</h3>' + \
                f'request form: {request.form} <br />' + \
                f'request body: {request.json if request.is_json else ""}'), 400
    elif error_info == error_info_type.JSON:
        return self_result.JsonResult(400, request.json if request.is_json else '',
                                      'please check your request data').as_res()
    elif error_info == error_info_type.XML:
        return self_result.XmlResult(400, request.json if request.is_json else '',
                                     'please check your request data').as_res()

# endregion



# coding: utf-8
import functools
import os
import sys
import re

from dophon import pre_boot

pre_boot.check_modules()

from dophon import properties, blue_print

from dophon_logger import *

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())

from werkzeug import _internal

_internal._logger = logger

from . import tools


def load_banner():
    """
    加载header-banner文件
    :return:
    """
    file_root = properties.project_root
    file_path = file_root + os.path.sep + 'header.txt'
    if os.path.exists(file_path):
        with open(file_path, encoding='utf8') as banner:
            for line in banner.readlines():
                sys.stdout.write(line)
    else:
        tools.show_banner()
    sys.stdout.flush()


def load_footer():
    """
    加载footer-banner文件
    :return:
    """
    file_root = properties.project_root
    file_path = file_root + os.path.sep + 'footer.txt'
    if os.path.exists(file_path):
        with open(file_path, encoding='utf8') as banner:
            for line in banner.readlines():
                sys.stdout.write(line)
    sys.stdout.flush()


def BeanScan(scan_path: list = []):
    def method(f):
        @functools.wraps(f)
        def method_args(*args, **kwargs):
            logger.debug('mapping beans')
            __scan_path = scan_path if scan_path else properties.components_path
            for bean_path in __scan_path:
                is_project_root = os.path.abspath(properties.project_root + bean_path) == os.path.abspath(
                    properties.project_root)
                if is_project_root:
                    # 若配置项目自身路径则提示警告
                    logger.warning(f'扫描路径(components_path)存在项目根路径的配置会导致项目异常启动,请注意')
                map_bean(bean_path)
            return f(*args, **kwargs)

        return method_args

    return method


def map_bean(bean_path: str, is_project_root: bool = False):
    __project_root = properties.project_root.replace("\\", "/")
    bean_dir = re.sub('(\\\\|/)', '', bean_path)
    # 不存在路由定义
    # print(f'bean_path: {bean_path}')
    try:
        for root, dirs, files in os.walk(f'{__project_root}{bean_path}'):
            relative_path = re.sub(f'{__project_root}', '', root)
            # 若处于路由定义则跳过
            # 相对路径为根路径也跳过
            # 强制指定为项目根路径也跳过
            if relative_path in properties.blueprint_path \
                    or relative_path == '/' \
                    or relative_path.endswith('__') \
                    or is_project_root:
                continue
            # 校验忽略注入的配置
            __pass_component_flag = False
            __uncomponents_path = [
                i.replace(f'{os.path.sep}', '')
                for i in (properties.uncomponents_path if hasattr(properties, 'uncomponents_path') else [])
            ]
            __uncomponents_path.append(f'/site-packages')
            __uncomponents_path.append(f'/.package_cache')
            for __up_items in __uncomponents_path:
                if relative_path.startswith(__up_items):
                    __pass_component_flag = True
                    break
            if __pass_component_flag:
                continue
            logger.debug(f'scan path {relative_path}')
            # exec(f'from {re.sub("/", ".", __bean_dir)} import {module_path.split(".")[-1]}')
            # print(f'{root} => {relative_path} => {dirs} => {files}')
            # print(__dirs)
            for file in files:
                file_short_name = os.path.basename(file).split(".")[0]
                if not file.endswith('.py') or re.match('__.+__', file_short_name):
                    # 不是python脚本文件
                    continue
                # 获取内部文件路径
                __file_path = f'{relative_path}/{file_short_name}'
                # 整理成包形式
                __file_package = re.sub('(\\\\|/)', '.', __file_path)
                # print(__file_package)
                __file_package_part = __file_package.split('.')[1:]
                # print(__file_package_part)
                __exec_code = f'from {".".join(__file_package_part[:-1])} import {__file_package_part[-1]}'
                # print(__exec_code)
                logger.debug(f'mapping bean {__file_package_part[-1]} {__exec_code}')
                try:
                    exec(__exec_code)
                except Exception as e:
                    logger.error(f'{__file_package_part[-1]} inject failed, reason: {e}')
        __import__(bean_dir, fromlist=bean_dir)
    except Exception as e:
        # raise e
        pass


@BeanScan()
def d_app(f):
    load_banner()

    @functools.wraps(f)
    def __d_app(*arg, **kwargs):
        return f(*arg, **kwargs) if callable(f) else None

    load_footer()
    return __d_app



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dophon',
 'dophon.annotation',
 'dophon.annotation.description',
 'dophon.annotation.request',
 'dophon.annotation.request.param_adapter',
 'dophon.annotation.response',
 'dophon.errors',
 'dophon.tools',
 'dophon.tools.framework_const',
 'dophon.tools.gc',
 'dophon.web_templates']

package_data = \
{'': ['*']}

install_requires = \
['dophon-logger',
 'dophon-manager',
 'dophon-properties',
 'flask',
 'tornado',
 'tqdm',
 'urllib3',
 'xmltodict>=0.12.0,<0.13.0']

setup_kwargs = {
    'name': 'dophon',
    'version': '1.3.6.post10',
    'description': 'dophon web framework like springboot',
    'long_description': None,
    'author': 'CallMeE',
    'author_email': 'ealohu@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# from dophon_properties import *
#
# get_properties(DOPHON)
from dophon import properties
from dophon_logger import *

"""
框架其余功能装饰器
"""
logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())


def logger_execute(*args, **kwargs):
    logger.info(f'{args} : {kwargs}')


def log_router(func=logger_execute):
    # 执行路由日志记录
    func()

    def r_method(f):
        def r_args(*args, **kwargs):
            f(*args, **kwargs)

        return r_args

    return r_method


def logs():
    print('logs!!!')


@log_router()
def test():
    print('ooo')


test()
